;----------------------------------------------------------;
; Melody Generator  (C)ChaN, 2005
;
; Additional code	(C)MoJo, 2010
;
;	PB3 - attention
;	PB0 - data


.include "tn45def.inc"	;This is included in "Atmel AVR Studio"
.include "avr.inc"
.include "mg.inc"

.def	_0	= r15
.def	_Sreg	= r14
.def	_Zreg	= r12
.def	_Yreg	= r10
.def	_TmrH	= r9
.def	_TmrL	= r8
.def	_TmrS	= r7

.def	decayspeed = r6		; MoJo decay speed

.equ	N_NOTE	= 6


;----------------------------------------------------------;
; Work Area

.dseg
	.org	RAMTOP
NoteIdx:.byte	1	; Note rotation index

Notes:	.byte	(2+3+1+1+1+1)*N_NOTE
.equ	ns_freq = 0	;Angular Speed
.equ	ns_rptr = 2	;Wave table read pointer (16.8 fraction)
.equ	ns_lvl = 5	;Level
.equ	ns_wrap = 6	;Loop Flag
.equ	ns_loop = 7	;Loop Count
.equ	ns_lp = 8	;Level Pointer
.equ	nsize = 9	;size of this structure


;----------------------------------------------------------;
; Program Code

.cseg
	; Interrupt Vectors (ATtiny45)
	rjmp	reset		; Reset
	rjmp	0		; INT0
	rjmp	0		; PCINT0
	rjmp	0		; TC1_COMA
	rjmp	0		; TC1_OVF
	rjmp	0		; TC0_OVF
	rjmp	0		; EE_RDY
	rjmp	0		; ANA_COMP
	rjmp	0		; ADC
	rjmp	0		; TC1_COMB
	rjmp	isr_tc0_coma	; TC0_COMA
;	rjmp	0		; TC0_COMB
;	rjmp	0		; WDT
;	rjmp	0		; USI_START
;	rjmp	0		; USI_OVF


;--------------------------------------------------------------------;
; Program Code

reset:
	clr	_0
	ldiw	X, RAMTOP		;Clear RAM
	ldi	AL, 0			;
	st	X+, _0			;
	dec	AL			;
	brne	PC-2			;/

;	outi	OSCCAL, 172		;Adjust OSCCAL if needed.

	outi	PORTB, 0b001101		;Initalize Port B
	outi	DDRB,  0b010010		;/

	outi	PLLCSR, 0b00000110	;Initialize TC1 in 250 kHz fast PWM mode.
	outi	TCCR1,  0b01100001	;Connect TC1 to OC1A
	outi	GTCCR,  0b01100000	;Connect TC1 to OC1B

	outi	OCR0A, 62		;Initalize TC0 in 32 kHz interval timer.
	outi	TCCR0A, 0b00000010
	outi	TCCR0B, 0b00000010
	outi	TIMSK, (1<<OCIE0A)

	ldi		AL, 6
	mov		decayspeed, AL

;rjmp count_pulses_done

wait_attention:
;ldiw	Z, score1*2
;rjmp	start_play
	; wait for attention line
	sbic	PINB, 3
	rjmp	wait_attention

	clr		AL					; pulse counter
count_pulses_h:
	sbic	PINB, 3				; attention
	rjmp	count_pulses_done
	sbic	PINB, 0				; data pulse
	rjmp	count_pulses_h
	inc		AL
count_pulses_l:
	sbic	PINB, 3				; attention
	rjmp	count_pulses_done
	sbis	PINB, 0				; data pulse
	rjmp	count_pulses_l
	rjmp	count_pulses_h

count_pulses_done:
		cpi		AL, 5			; sanity check
		brge	wait_attention

		;ldi		AL, 3
		;andi	AL, 0b00000011

		ldi		ZH, HIGH(melody_jump)
		ldi		ZL, LOW(melody_jump)
		add		ZL, AL
		clr		AL
		adc		ZH, AL
		ijmp
melody_jump:
		rjmp	melody1
		rjmp	melody2
		rjmp	melody3
		rjmp	melody4
		rjmp	melody5

melody1:
	ldiw	Z, score1*2
	rjmp	start_play
melody2:
	ldiw	Z, score2*2
	rjmp	start_play
melody3:
	ldiw	Z, score3*2
	rjmp	start_play
melody4:
	ldiw	Z, score4*2
	rjmp	start_play
melody5:
	ldiw	Z, score5*2
	rjmp	start_play

start_play:
	lpm		decayspeed, Z+		; skip initial 0
	lpm		decayspeed, Z+
	cli
	clrw	_Tmr
	clr	_TmrS
	sei

pl_next:
	lpmw	B, Z+
pl_next_loop:
	sbis	PINB, 3
	rjmp	stop_play
	 rcall	drv_decay
	cli
	cpw	_Tmr, B
	sei
	;brcs	PC-5
	brcs	pl_next_loop

pl_note:
	lpm	CL, Z+
	cpi	CL, EoS
	;breq	start_play
	breq	stop_play
	mov	AL, CL
	 rcall	note_on
	andi	CL, en
	breq	pl_note
	rjmp	pl_next


stop_play:
	;rjmp	stop_play
	rjmp	wait_attention

;--------------------------------------------------------------------;
; Note ON
;
;Call: AL[6:0] = key number

note_on:
	pushw	Z

	mov	ZL, AL
	lsl	ZL
	clr	ZH
	addiw	Z, tbl_pitch*2
	lpmw	A, Z+

	lds	YL, NoteIdx
	addi	YL, 9
	cpi	YL, 9*N_NOTE
	brcs	PC+2
	clr	YL
	sts	NoteIdx, YL
	clr	YH
	addiw	Y, Notes

	ldiw	B, wt_attack*2
	cli
	stdw	Y+ns_freq, A
	stdw	Y+ns_rptr+1, B
	sei
	stdi	Y+ns_lvl, 255
	std	Y+ns_wrap, AL
	std	Y+ns_loop, _0
	std	Y+ns_lp, _0

	popw	Z
	ret


;--------------------------------------------------------------------;
; Decay envelope generation

drv_decay:
	pushw	Z
	ldiw	Y, Notes
dd_lp:
	ldd	AL, Y+ns_wrap	;Has sustain loop not wrapped?
	ldi	AH, 255		;
	cp	AL, AH		;
	breq	dd_nxt		;/
	std	Y+ns_wrap, AH	;Clear wrapped flag.
	ldd	AL, Y+ns_loop
	;inc	AL
	add		AL, decayspeed
	cpi	AL, 12
	brcs	PC+2
	ldi	AL, 0
	std	Y+ns_loop, AL
	brcs	dd_nxt
	ldd	ZL, Y+ns_lp
	inc	ZL
	breq	dd_nxt
	std	Y+ns_lp, ZL
	clr	ZH
	addiw	Z, envelope*2
	lpm	AL, Z
	std	Y+ns_lvl, AL
dd_nxt:	adiw	YL, 9
	cpi	YL, low(Notes+nsize*N_NOTE)
	brne	dd_lp

	popw	Z
	ret



;--------------------------------------------------------------------;
; 32 kHz wave form synthesising interrupt


isr_tc0_coma:
	in	_Sreg, SREG		;Save regs...
	movw	_Zreg, ZL		;
	movw	_Yreg, YL		;/

	ldiw	Y, Notes		;Process all notes
	clrw	T2			;Clear accumlator
tone_lp:
	ldd	EH, Y+ns_rptr		;Load wave table pointer
	lddw	Z, Y+ns_rptr+1		;/
	lpm	EL, Z			;Get a sample
	lddw	T4, Y+ns_freq		;Load angular speed
	add	EH, T4L			;Increase wave table ptr (next angle)
	adc	ZL, T4H			;
	adc	ZH, _0			;/
	cpi	ZH, high(wt_end*2)	;Repeat sustain area
	brcs	PC+4			;
	subiw	Z, (wt_end-wt_loop)*2	;
	std	Y+ns_wrap, _0		;/
	std	Y+ns_rptr, EH		;Save wave table ptr
	stdw	Y+ns_rptr+1, Z		;/
	ldd	EH, Y+ns_lvl		;Apply envelope curve
	MULT				;/
	addw	T2, T0			;Add the sample to accumlator
	adiw	YL, 9			;Next note
	cpi	YL, low(Notes+nsize*N_NOTE);
	brne	tone_lp			;/

	asrw	T2			;Divide it by 4
	asrw	T2			;/
	ldiw	E, 253			;Clip it between -255 to 253
	cpw	T2, E			;
	brlt	PC+2			;
	movw	T2L, EL			;
	ldiw	E, -255			;
	cpw	T2, E			;
	brge	PC+2			;
	movw	T2L, EL			;/
	asrw	T2			;Set it to PWM modulator
	ror	T2H			;
	mov	EL, T2L			;
	subi	EL, 0x80		;
	mov	EH, EL			;
	com	EH			;
	sbrc	T2H, 7			;
	inc	EL			;
	out	OCR1A, EL		;
	out	OCR1B, EH		;/

	sec				;Increment sequense timer
	adc	_TmrS, _0		;
	adc	_TmrL, _0		;
	adc	_TmrH, _0		;/

	movw	ZL, _Zreg		;Restore regs...
	movw	YL, _Yreg		;
	out	SREG, _Sreg		;/

	reti



;--------------------------------------------------------------------;
; Score table
;--------------------------------------------------------------------;

score1:
.db 0, 1	; decay speed
.include "melody.asm"

score2:
.db 0, 1	; decay speed
.include "melody2.asm"

score3:
.db 0, 1	; decay speed
.include "melody3.asm"

score4:
.db 0, 4	; decay speed
.include "melody4.asm"

score5:
.db 0, 2	; decay speed
.include "melody5.asm"

;--------------------------------------------------------------------;
; Pitch number to angular speed conversion table
;--------------------------------------------------------------------;
;Since sustain area of wave table, a cycle of fundamental frequency, is sampled
;in 128 points, the base frequency becomes 32000/128 = 250 Hz. The wave table
;lookup pointer, 16.8 fraction, is increased every sample by these 8.8 fractional
;angular speed values.

tbl_pitch: ;  A     B     H     C    Cis    D    Dis    E     F    Fis    G    Gis 
	.dw  225,  239,  253,  268,  284,  301,  319,  338,  358,  379,  401,  425 ; 220Hz..
	.dw  451,  477,  506,  536,  568,  601,  637,  675,  715,  758,  803,  851 ; 440Hz..
	.dw  901,  955, 1011, 1072, 1135, 1203, 1274, 1350, 1430, 1515, 1606, 1701 ; 880Hz..
	.dw 1802, 1909, 2023, 2143, 2271, 2406, 2549, 2700, 2861, 3031, 3211, 3402 ; 1760Hz..
	.dw 3604, 3818, 4046, 4286, 4542, 4812, 5098, 5400			   ; 3520Hz


;--------------------------------------------------------------------;
; Envelope Table
;--------------------------------------------------------------------;

envelope:
	.db 255,252,250,247,245,243,240,238,235,233,231,228,226,224,222,219
	.db 217,215,213,211,209,207,205,203,201,199,197,195,193,191,189,187
	.db 185,183,182,180,178,176,174,173,171,169,168,166,164,163,161,159
	.db 158,156,155,153,152,150,149,147,146,144,143,141,140,139,137,136
	.db 134,133,132,130,129,128,127,125,124,123,122,120,119,118,117,116
	.db 115,113,112,111,110,109,108,107,106,105,104,103,102,101,100,99
	.db 98,97,96,95,94,93,92,91,90,89,88,87,87,86,85,84
	.db 83,82,82,81,80,79,78,78,77,76,75,75,74,73,72,72
	.db 71,70,69,69,68,67,67,66,65,65,64,64,63,62,62,61
	.db 60,60,59,59,58,57,57,56,56,55,55,54,54,53,53,52
	.db 51,51,50,50,49,49,48,48,48,47,47,46,46,45,45,44
	.db 44,43,43,43,42,42,41,40,40,39,39,38,38,37,37,36
	.db 35,35,34,34,33,33,32,31,31,30,30,29,29,28,28,27
	.db 26,26,25,25,24,24,23,22,22,21,21,20,20,19,19,18
	.db 17,17,16,16,15,15,14,13,13,12,12,11,11,10,10,9
	.db 8,8,7,7,6,6,5,4,4,3,3,2,2,1,1,0

;.include "env.inc"

;--------------------------------------------------------------------;
; Wave Table
;--------------------------------------------------------------------;
; 8bit, 32 ksps, 250 Hz fundamental frequency

	//.org	3072/2	; Bottom stored

/*
wt_attack: ; Attack area
	.db 5, 8, 9, 11, 12, 15, 17, 19, 21, 23, 24, 26, 28, 30, 31, 33
	.db 35, 36, 38, 40, 41, 43, 44, 45, 47, 49, 50, 52, 53, 55, 56, 58
	.db 59, 59, 59, 59, 59, 58, 58, 57, 56, 55, 54, 53, 51, 50, 48, 47
	.db 46, 44, 43, 42, 41, 40, 39, 38, 38, 37, 36, 35, 34, 33, 32, 31
	.db 29, 28, 26, 25, 23, 21, 19, 18, 16, 14, 11, 9, 8, 6, 4, 3
	.db 1, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2
	.db 2, 2, 2, 1, 1, 0, 0, -1, -2, -3, -5, -6, -8, -9, -11, -13
	.db -15, -18, -20, -22, -25, -27, -29, -30, -32, -33, -35, -36, -36, -37, -37, -37
	.db -36, -35, -33, -31, -29, -26, -23, -19, -15, -11, -7, -3, 1, 5, 10, 15
	.db 19, 24, 28, 32, 36, 39, 42, 44, 46, 47, 48, 49, 49, 49, 49, 48
	.db 47, 46, 45, 44, 42, 41, 39, 37, 35, 32, 30, 27, 25, 22, 19, 16
	.db 14, 11, 8, 5, 3, 0, -1, -3, -4, -6, -7, -8, -9, -10, -10, -10
	.db -10, -10, -9, -9, -8, -8, -7, -6, -5, -5, -4, -3, -2, -1, 0, 0
	.db 0, 1, 2, 3, 4, 5, 6, 7, 7, 8, 8, 7, 7, 6, 5, 4
	.db 2, 0, -1, -3, -6, -8, -11, -14, -18, -21, -24, -28, -31, -34, -38, -41
	.db -44, -47, -50, -53, -55, -57, -59, -61, -63, -64, -65, -66, -67, -67, -68, -68
	.db -68, -68, -67, -67, -66, -65, -65, -64, -63, -62, -61, -60, -59, -58, -57, -56
	.db -55, -55, -54, -53, -53, -52, -52, -52, -52, -52, -52, -53, -54, -55, -56, -57
	.db -59, -60, -62, -64, -65, -67, -69, -71, -72, -73, -75, -76, -77, -78, -78, -79
	.db -79, -79, -79, -79, -79, -78, -77, -76, -75, -74, -73, -71, -70, -68, -66, -64
	.db -61, -59, -56, -53, -50, -47, -44, -40, -37, -33, -29, -25, -22, -18, -14, -10
	.db -7, -3, 0, 2, 5, 8, 10, 12, 13, 15, 15, 16, 16, 16, 15, 14
	.db 13, 12, 10, 8, 6, 4, 2, 0, -2, -5, -8, -11, -14, -17, -20, -23
	.db -26, -30, -33, -36, -39, -42, -45, -48, -50, -53, -55, -58, -59, -61, -63, -64
	.db -65, -66, -67, -67, -67, -68, -68, -67, -67, -67, -67, -67, -66, -66, -66, -66
	.db -66, -66, -66, -66, -66, -67, -67, -68, -68, -69, -69, -70, -70, -71, -72, -72
	.db -73, -73, -74, -74, -75, -75, -75, -76, -76, -76, -77, -77, -77, -78, -78, -79
	.db -79, -79, -79, -79, -79, -79, -78, -77, -76, -75, -74, -72, -70, -67, -65, -61
	.db -58, -54, -50, -46, -42, -37, -33, -28, -24, -19, -14, -10, -5, -1, 2, 6
	.db 10, 14, 18, 21, 24, 27, 29, 32, 34, 36, 37, 39, 40, 41, 41, 42
	.db 42, 41, 41, 40, 39, 38, 37, 35, 33, 31, 29, 27, 24, 22, 19, 17
	.db 15, 13, 11, 9, 7, 6, 4, 3, 3, 2, 2, 1, 1, 2, 2, 2
	.db 3, 3, 4, 5, 6, 6, 7, 8, 9, 9, 10, 11, 11, 12, 13, 13
	.db 14, 15, 15, 16, 17, 18, 19, 20, 20, 21, 22, 22, 23, 23, 23, 23
	.db 22, 22, 21, 20, 19, 18, 16, 15, 13, 12, 10, 8, 7, 5, 4, 2
	.db 1, 0, 0, 0, 0, -1, -1, 0, 0, 0, 1, 2, 4, 5, 7, 9
	.db 11, 13, 15, 18, 21, 23, 26, 29, 32, 36, 39, 42, 46, 49, 53, 56
	.db 60, 63, 66, 69, 72, 75, 78, 80, 82, 84, 85, 86, 87, 87, 87, 87
	.db 87, 86, 85, 84, 83, 82, 80, 79, 78, 77, 76, 76, 75, 74, 74, 73
	.db 73, 73, 72, 72, 72, 72, 72, 72, 72, 72, 73, 73, 73, 74, 75, 75
	.db 76, 77, 78, 80, 81, 82, 84, 86, 87, 89, 91, 92, 94, 95, 97, 98
	.db 99, 100, 101, 102, 102, 102, 102, 102, 101, 101, 100, 99, 98, 96, 95, 93
	.db 91, 88, 86, 84, 81, 78, 76, 73, 70, 66, 63, 59, 56, 52, 49, 45
	.db 41, 37, 34, 30, 26, 23, 19, 16, 13, 10, 7, 5, 3, 1, 0, -1
	.db -1, -2, -2, -2, -2, -2, -1, -1, 0, 0, 1, 2, 2, 3, 4, 4
	.db 4, 5, 5, 4, 4, 4, 3, 3, 2, 2, 1, 0, 0, -1, -1, -2
	.db -3, -4, -4, -5, -6, -7, -8, -9, -9, -10, -11, -12, -12, -13, -14, -14
	.db -14, -14, -15, -14, -14, -13, -12, -11, -10, -9, -7, -5, -3, -1, 0, 3
	.db 6, 8, 11, 14, 16, 19, 22, 25, 28, 31, 33, 36, 38, 41, 43, 46
	.db 48, 50, 52, 54, 56, 58, 60, 61, 63, 64, 65, 65, 66, 66, 66, 65
	.db 65, 63, 62, 60, 58, 55, 53, 49, 46, 42, 38, 34, 29, 25, 20, 15
	.db 11, 6, 1, -2, -6, -10, -14, -17, -21, -24, -26, -29, -31, -34, -35, -37
	.db -39, -40, -42, -43, -44, -45, -46, -47, -48, -49, -50, -50, -51, -52, -52, -53
	.db -54, -54, -55, -56, -56, -57, -57, -58, -59, -60, -61, -62, -63, -65, -66, -67
	.db -69, -71, -72, -74, -76, -77, -79, -80, -81, -83, -84, -85, -86, -86, -87, -87
	.db -87, -87, -87, -86, -86, -85, -84, -83, -81, -80, -78, -76, -75, -73, -71, -69
	.db -67, -64, -62, -59, -57, -54, -51, -48, -44, -41, -37, -34, -30, -26, -23, -19
	.db -15, -11, -8, -5, -1, 1, 3, 6, 8, 10, 12, 13, 14, 15, 15, 15
	.db 15, 15, 14, 13, 11, 10, 8, 6, 3, 1, 0, -3, -5, -8, -11, -13
	.db -16, -18, -21, -23, -25, -27, -29, -31, -33, -34, -36, -37, -38, -39, -40, -40
	.db -41, -41, -41, -41, -41, -40, -40, -39, -39, -38, -37, -37, -36, -36, -35, -35
	.db -35, -35, -35, -36, -36, -37, -38, -39, -40, -41, -43, -44, -45, -47, -48, -50
	.db -51, -52, -53, -54, -55, -56, -57, -58, -59, -60, -61, -62, -63, -64, -66, -67
	.db -68, -69, -70, -72, -73, -74, -75, -75, -76, -76, -77, -77, -76, -76, -75, -75
	.db -74, -72, -71, -69, -68, -66, -63, -61, -59, -56, -53, -51, -48, -45, -42, -40
	.db -37, -34, -32, -29, -27, -25, -23, -21, -19, -17, -16, -15, -14, -13, -13, -12
	.db -12, -12, -12, -12, -12, -13, -13, -14, -15, -16, -17, -18, -19, -20, -21, -22
	.db -23, -24, -25, -25, -25, -25, -25, -25, -24, -23, -22, -20, -18, -16, -14, -11
	.db -9, -6, -3, -1, 1, 4, 6, 9, 11, 13, 15, 17, 19, 21, 22, 23
	.db 25, 26, 27, 28, 29, 30, 31, 31, 32, 32, 33, 33, 34, 34, 34, 34
	.db 33, 33, 32, 31, 30, 29, 27, 26, 24, 22, 20, 18, 16, 13, 11, 9
	.db 6, 4, 2, 0, -1, -3, -4, -6, -7, -9, -10, -11, -11, -12, -13, -13
	.db -13, -13, -13, -13, -12, -12, -11, -10, -9, -7, -6, -4, -3, -1, 0, 2
	.db 4, 6, 8, 10, 12, 14, 15, 17, 18, 19, 20, 21, 22, 22, 23, 23
	.db 22, 22, 22, 21, 21, 20, 19, 19, 18, 17, 17, 17, 17, 17, 17, 18
	.db 19, 20, 21, 23, 25, 27, 29, 32, 34, 37, 39, 42, 45, 48, 51, 54
	.db 57, 60, 63, 66, 69, 72, 75, 78, 82, 85, 88, 91, 94, 98, 101, 104
	.db 107, 109, 112, 115, 117, 119, 121, 123, 124, 125, 126, 127, 127, 127, 127, 127
	.db 127, 126, 125, 124, 122, 121, 119, 117, 115, 113, 110, 108, 105, 102, 99, 96
	.db 92, 89, 85, 81, 77, 74, 70, 66, 62, 58, 55, 51, 48, 45, 42, 40
	.db 38, 36, 34, 33, 32, 31, 30, 29, 29, 29, 28, 28, 28, 28, 28, 27
	.db 27, 26, 26, 25, 24, 23, 22, 20, 19, 17, 15, 14, 12, 10, 8, 6
	.db 4, 2, 0, -1, -3, -5, -7, -9, -10, -12, -13, -14, -16, -17, -18, -18
	.db -19, -20, -20, -20, -20, -19, -19, -18, -17, -16, -14, -13, -11, -9, -7, -4
	.db -2, 0, 2, 5, 8, 11, 13, 16, 19, 21, 24, 26, 28, 30, 32, 34
	.db 36, 38, 40, 41, 43, 44, 46, 47, 49, 50, 51, 53, 54, 55, 55, 56
	.db 56, 56, 56, 56, 55, 54, 53, 51, 49, 47, 45, 43, 40, 37, 34, 31
	.db 28, 25, 21, 18, 15, 13, 10, 7, 5, 3, 1, 0, -1, -3, -4, -5
	.db -5, -6, -7, -7, -7, -7, -8, -8, -8, -8, -8, -8, -8, -8, -8, -8
	.db -8, -9, -9, -9, -10, -11, -11, -12, -13, -14, -16, -17, -19, -21, -23, -25
	.db -27, -30, -32, -35, -38, -40, -43, -46, -49, -52, -54, -57, -59, -62, -64, -66
	.db -67, -69, -70, -71, -72, -73, -74, -74, -75, -75, -75, -75, -74, -74, -74, -74
	.db -73, -73, -73, -72, -71, -71, -70, -70, -69, -68, -67, -66, -65, -64, -62, -61
	.db -59, -57, -56, -54, -52, -51, -49, -47, -46, -44, -43, -42, -40, -40, -39, -39
	.db -38, -38, -38, -39, -39, -40, -41, -42, -43, -44, -45, -46, -48, -49, -50, -52
	.db -53, -54, -55, -56, -57, -58, -58, -59, -59, -59, -59, -59, -58, -57, -56, -55
	.db -53, -51, -50, -47, -45, -43, -40, -38, -35, -33, -30, -28, -26, -23, -21, -20
	.db -18, -16, -15, -14, -14, -13, -13, -13, -14, -14, -15, -16, -17, -19, -20, -22
	.db -24, -26, -28, -30, -32, -33, -35, -37, -39, -41, -43, -44, -46, -47, -49, -51
	.db -52, -54, -56, -57, -59, -60, -62, -64, -65, -67, -68, -70, -71, -73, -74, -75
	.db -76, -77, -77, -78, -78, -79, -79, -79, -79, -79, -78, -78, -77, -77, -76, -75
	.db -74, -73, -72, -71, -70, -69, -68, -67, -66, -65, -64, -63, -62, -61, -60, -59
	.db -59, -58, -58, -58, -58, -58, -58, -59, -59, -60, -60, -61, -62, -63, -63, -64
	.db -64, -65, -65, -65, -64, -64, -63, -61, -60, -58, -55, -53, -50, -47, -43, -40
	.db -36, -32, -28, -23, -19, -15, -11, -6, -2, 1, 5, 8, 12, 15, 19, 22
	.db 24, 27, 29, 31, 33, 35, 36, 38, 39, 40, 40, 41, 41, 41, 42, 42
	.db 42, 41, 41, 40, 40, 39, 38, 37, 36, 35, 33, 32, 30, 29, 27, 25
	.db 23, 22, 20, 18, 16, 15, 13, 11, 10, 8, 7, 5, 4, 3, 1, 0
	.db 0, -1, -2, -3, -4, -5, -6, -7, -8, -8, -8, -9, -8, -8, -8, -7
	.db -7, -6, -5, -4, -3, -2, -1, 0, 0, 0, 1, 2, 2, 2, 2, 2
	.db 2, 1, 1, 0, 0, -1, -2, -3, -4, -6, -7, -7, -8, -9, -9, -9
	.db -9, -9, -8, -7, -6, -5, -3, -1, 0, 2, 5, 8, 11, 14, 17, 21
	.db 24, 28, 31, 35, 38, 42, 46, 49, 53, 56, 60, 63, 67, 70, 73, 76
	.db 79, 82, 85, 88, 90, 92, 94, 96, 98, 99, 101, 102, 103, 103, 104, 104
	.db 104, 105, 104, 104, 104, 104, 103, 102, 102, 101, 100, 99, 98, 98, 96, 95
	.db 94, 93, 92, 90, 89, 87, 86, 84, 83, 81, 79, 77, 76, 74, 72, 71
	.db 69, 68, 67, 66, 65, 65, 65, 65, 65, 65, 65, 66, 66, 67, 68, 68
	.db 69, 69, 69, 70, 70, 70, 69, 69, 68, 67, 66, 65, 64, 62, 61, 59
	.db 57, 55, 52, 50, 48, 45, 43, 41, 38, 36, 33, 31, 28, 26, 24, 21
	.db 20, 17, 17, 11

wt_loop: ; Sustain area
	.db -2, -5, -6, -8, -10, -11, -13, -14, -15, -16, -17, -18, -19, -19, -20, -20
	.db -20, -19, -19, -18, -18, -16, -15, -14, -12, -10, -8, -5, -3, 0, 1, 4
	.db 7, 9, 12, 15, 18, 20, 23, 25, 27, 29, 32, 33, 35, 37, 39, 40
	.db 42, 44, 45, 47, 48, 50, 51, 52, 53, 54, 55, 56, 56, 56, 56, 56
	.db 55, 55, 53, 52, 50, 48, 46, 44, 41, 38, 35, 32, 29, 26, 23, 20
	.db 17, 14, 11, 8, 6, 4, 2, 0, -1, -2, -3, -4, -5, -6, -6, -7
	.db -7, -7, -8, -8, -8, -8, -8, -8, -8, -8, -8, -8, -8, -9, -9, -9
	.db -10, -10, -11, -12, -13, -14, -15, -17, -18, -20, -22, -24, -26, -29, -31, -34
	.db -36, -39, -42, -45, -48, -51, -53, -56, -58, -61, -63, -65, -67, -68, -70, -71
	.db -72, -73, -73, -74, -74, -75, -75, -75, -75, -74, -74, -74, -74, -73, -73, -72
	.db -72, -71, -71, -70, -69, -68, -67, -66, -65, -64, -63, -61, -60, -58, -56, -55
	.db -53, -51, -50, -48, -46, -45, -43, -42, -41, -40, -39, -39, -38, -38, -38, -39
	.db -39, -40, -40, -41, -42, -43, -45, -46, -47, -48, -50, -51, -52, -53, -55, -56
	.db -57, -57, -58, -59, -59, -59, -59, -59, -58, -58, -57, -55, -54, -52, -50, -48
	.db -46, -44, -41, -39, -36, -34, -31, -29, -27, -24, -22, -20, -19, -17, -16, -15
	.db -14, -13, -13, -13, -13, -14, -15, -16, -17, -18, -20, -21, -23, -25, -27, -29
	.db -31, -33, -34, -36, -38, -40, -42, -43, -45, -47, -48, -50, -52, -53, -55, -56
	.db -58, -60, -61, -63, -65, -66, -68, -69, -71, -72, -73, -74, -75, -76, -77, -78
	.db -78, -79, -79, -79, -79, -79, -78, -78, -78, -77, -76, -76, -75, -74, -73, -72
	.db -71, -70, -69, -68, -66, -65, -64, -63, -62, -61, -60, -60, -59, -58, -58, -58
	.db -58, -58, -58, -58, -59, -59, -60, -61, -62, -62, -63, -64, -64, -65, -65, -65
	.db -64, -64, -63, -62, -60, -59, -56, -54, -51, -48, -45, -41, -38, -34, -30, -25
	.db -21, -17, -12, -8, -4, 0, 3, 7, 10, 14, 17, 20, 23, 26, 28, 30
	.db 32, 34, 36, 37, 38, 39, 40, 41, 41, 41, 42, 42, 42, 41, 41, 41
	.db 40, 39, 38, 37, 36, 35, 34, 32, 31, 29, 28, 26, 24, 22, 21, 19
	.db 17, 15, 14, 12, 11, 9, 8, 6, 5, 3, 2, 1, 0, -1, -2, -3
	.db -4, -5, -6, -7, -7, -8, -8, -8, -8, -8, -8, -8, -7, -6, -6, -5
	.db -4, -3, -2, -1, 0, 0, 1, 1, 2, 2, 3, 2, 2, 2, 1, 0
	.db 0, -1, -2, -3, -4, -5, -6, -7, -8, -8, -9, -9, -9, -9, -8, -8
	.db -7, -5, -4, -2, 0, 1, 4, 7, 9, 13, 16, 19, 22, 26, 30, 33
	.db 37, 40, 44, 48, 51, 55, 58, 62, 65, 69, 72, 75, 78, 81, 84, 87
	.db 89, 91, 93, 95, 97, 99, 100, 101, 102, 103, 104, 104, 104, 105, 104, 104
	.db 104, 104, 103, 103, 102, 101, 101, 100, 99, 98, 97, 96, 95, 94, 92, 91
	.db 90, 88, 87, 85, 83, 82, 80, 78, 76, 75, 73, 71, 70, 69, 68, 67
	.db 66, 65, 65, 65, 65, 65, 65, 65, 66, 67, 67, 68, 68, 69, 69, 70
	.db 70, 70, 70, 69, 69, 68, 67, 66, 64, 63, 61, 60, 58, 56, 53, 51
	.db 49, 47, 44, 42, 39, 37, 34, 32, 29, 27, 25, 22, 21, 18, 18, 8
	.db -1, 0
wt_end:
*/

// Second tone table //


wt_attack: ; Attack area
	.db -4, -4, -4, -7, -10, -13, -13, -9, 0, 6, 8, 5, 1, -1, -3, -2
	.db 0, 3, 5, 4, -2, -14, -30, -45, -54, -53, -47, -37, -22, -7, 6, 20
	.db 35, 47, 57, 63, 68, 71, 68, 61, 50, 37, 25, 17, 14, 16, 18, 18
	.db 17, 18, 19, 20, 21, 22, 23, 22, 17, 9, -3, -18, -35, -47, -53, -52
	.db -47, -39, -30, -23, -18, -14, -9, -4, 1, 8, 15, 20, 22, 20, 15, 11
	.db 8, 7, 9, 14, 19, 22, 23, 23, 20, 15, 12, 11, 13, 15, 15, 9
	.db 1, -8, -17, -24, -24, -22, -18, -12, -8, -7, -12, -22, -32, -39, -40, -37
	.db -33, -29, -28, -30, -33, -33, -31, -26, -17, -3, 10, 21, 27, 26, 21, 16
	.db 12, 9, 7, 0, -10, -25, -40, -53, -63, -68, -65, -57, -46, -35, -27, -24
	.db -25, -27, -29, -28, -26, -27, -30, -36, -44, -52, -60, -63, -61, -54, -41, -23
	.db -3, 15, 31, 45, 57, 68, 77, 83, 86, 85, 77, 67, 56, 43, 32, 24
	.db 22, 24, 30, 35, 39, 41, 42, 42, 44, 46, 49, 49, 47, 43, 34, 20
	.db 4, -10, -20, -24, -23, -19, -13, -7, -3, 0, 6, 14, 22, 31, 39, 46
	.db 49, 48, 44, 39, 35, 35, 38, 41, 42, 40, 34, 28, 21, 16, 13, 12
	.db 12, 10, 7, 1, -7, -18, -28, -33, -34, -31, -29, -27, -27, -27, -30, -32
	.db -35, -36, -37, -37, -36, -35, -33, -32, -30, -24, -15, -3, 8, 21, 32, 41
	.db 47, 52, 54, 53, 50, 46, 42, 35, 25, 13, 0, -11, -21, -26, -26, -22
	.db -18, -12, -7, -4, -5, -10, -16, -22, -27, -33, -40, -49, -59, -71, -82, -88
	.db -89, -85, -78, -66, -53, -39, -28, -17, -8, 0, 9, 16, 20, 18, 11, 0
	.db -12, -23, -33, -39, -40, -40, -38, -36, -34, -34, -34, -33, -31, -27, -24, -24
	.db -27, -32, -39, -47, -54, -59, -61, -61, -59, -55, -49, -43, -35, -26, -14, -1
	.db 12, 24, 35, 43, 49, 54, 58, 62, 66, 70, 74, 78, 81, 82, 81, 78
	.db 73, 69, 65, 63, 61, 59, 55, 51, 47, 42, 36, 32, 30, 29, 30, 31
	.db 31, 28, 24, 18, 15, 13, 10, 7, 5, 4, 3, 2, 3, 7, 14, 25
	.db 37, 50, 60, 64, 66, 65, 64, 62, 58, 54, 47, 38, 27, 14, 1, -9
	.db -18, -22, -22, -19, -16, -13, -11, -10, -10, -11, -12, -14, -20, -27, -36, -45
	.db -54, -62, -67, -67, -63, -57, -48, -38, -27, -15, -4, 8, 21, 32, 41, 47
	.db 50, 49, 45, 39, 31, 22, 15, 9, 5, 3, 2, 1, 0, 0, 0, -1
	.db -4, -6, -8, -12, -16, -23, -31, -41, -51, -59, -64, -68, -72, -74, -74, -74
	.db -74, -72, -67, -60, -53, -44, -36, -30, -28, -28, -28, -25, -21, -18, -15, -12
	.db -13, -15, -20, -25, -30, -35, -39, -40, -42, -46, -50, -55, -57, -58, -57, -54
	.db -50, -47, -45, -45, -44, -44, -44, -43, -41, -38, -37, -36, -34, -32, -29, -23
	.db -14, -1, 12, 27, 42, 57, 71, 82, 91, 98, 102, 104, 103, 101, 97, 90
	.db 83, 75, 69, 64, 61, 59, 59, 61, 64, 67, 70, 72, 71, 70, 68, 64
	.db 59, 51, 41, 30, 19, 8, 1, -1, -1, 2, 8, 17, 26, 33, 39, 45
	.db 53, 61, 67, 71, 72, 69, 63, 54, 44, 34, 26, 19, 14, 11, 7, 3
	.db 0, -3, -3, -3, -2, -2, -5, -10, -17, -24, -31, -38, -43, -47, -48, -49
	.db -52, -54, -55, -55, -53, -48, -40, -31, -22, -14, -6, 0, 6, 11, 16, 22
	.db 27, 31, 34, 35, 36, 34, 31, 27, 23, 17, 11, 4, -2, -8, -14, -18
	.db -22, -26, -28, -29, -29, -30, -32, -35, -39, -44, -50, -56, -61, -65, -69, -72
	.db -75, -77, -79, -81, -80, -73, -63, -51, -38, -26, -18, -12, -8, -7, -7, -9
	.db -13, -17, -23, -31, -42, -52, -61, -66, -68, -67, -64, -61, -58, -55, -51, -48
	.db -45, -41, -38, -36, -37, -40, -46, -52, -58, -61, -59, -54, -47, -39, -28, -16
	.db -2, 12, 26, 41, 56, 69, 80, 88, 94, 96, 96, 95, 93, 90, 87, 83
	.db 80, 79, 78, 77, 77, 78, 78, 79, 80, 80, 80, 77, 72, 66, 59, 51
	.db 44, 39, 35, 31, 29, 27, 25, 22, 20, 19, 21, 26, 31, 37, 43, 47
	.db 49, 51, 52, 54, 56, 57, 58, 58, 56, 50, 42, 35, 28, 21, 15, 9
	.db 2, -6, -15, -22, -27, -30, -31, -30, -27, -25, -25, -27, -32, -36, -40, -43
	.db -45, -46, -49, -52, -53, -54, -53, -50, -44, -35, -23, -11, 1, 14, 24, 33
	.db 40, 45, 48, 48, 43, 36, 28, 18, 8, 0, -8, -14, -18, -20, -21, -20
	.db -20, -20, -19, -20, -21, -23, -26, -30, -35, -42, -50, -60, -70, -79, -87, -90
	.db -89, -85, -78, -70, -61, -53, -45, -38, -30, -23, -17, -13, -10, -11, -15, -22
	.db -29, -35, -39, -43, -46, -49, -53, -57, -59, -60, -58, -56, -52, -49, -46, -46
	.db -49, -53, -56, -58, -58, -56, -53, -51, -50, -48, -45, -41, -36, -29, -20, -9

wt_loop: ; Sustain area
	.db -2, 5, 13, 20, 27, 33, 38, 43, 48, 51, 55, 58, 61, 64, 68, 71
	.db 75, 79, 83, 87, 91, 94, 97, 98, 99, 99, 97, 94, 91, 86, 80, 74
	.db 68, 61, 55, 49, 45, 41, 38, 36, 35, 35, 36, 37, 38, 40, 41, 43
	.db 43, 43, 43, 42, 40, 38, 36, 34, 32, 31, 30, 30, 30, 30, 31, 33
	.db 34, 35, 36, 36, 35, 33, 31, 28, 24, 19, 14, 9, 3, -1, -6, -10
	.db -13, -15, -16, -16, -15, -13, -10, -7, -4, 0, 2, 5, 8, 10, 11, 12
	.db 12, 11, 10, 9, 8, 6, 5, 5, 4, 4, 5, 5, 5, 5, 5, 3
	.db 1, -1, -5, -10, -16, -23, -31, -39, -47, -55, -63, -70, -76, -82, -86, -89
	.db -91, -92, -92, -91, -90, -88, -86, -84, -83, -82, -81, -81, -81, -81, -81, -81
	.db -81, -81, -80, -78, -76, -73, -69, -65, -61, -57, -52, -49, -45, -43, -41, -40
	.db -40, -40, -41, -43, -44, -45, -46, -46, -45, -44, -41, -37, -31, -25, -18, -10
wt_end:


/*
wt_attack: ; Attack area
	.db 0, 0, 0, 0, 0, 0, -1, -2, -2, -3, -2, -2, -1, 0, 0, 0
	.db 0, 0, -1, -2, -3, -3, -4, -4, -3, -2, -1, 0, 0, 1, 0, 0
	.db 0, -1, -2, -3, -3, -2, 0, 0, 2, 4, 5, 5, 5, 3, 1, 0
	.db -2, -3, -4, -3, -2, 0, 1, 2, 3, 2, 0, -2, -6, -11, -15, -18
	.db -19, -19, -16, -11, -5, 1, 8, 15, 20, 23, 24, 23, 20, 17, 13, 10
	.db 7, 6, 6, 8, 10, 13, 16, 18, 20, 20, 20, 19, 18, 18, 18, 18
	.db 19, 21, 24, 26, 27, 28, 27, 25, 22, 17, 11, 5, 0, -5, -9, -12
	.db -13, -13, -11, -9, -5, -1, 1, 4, 6, 6, 4, 0, -5, -12, -21, -30
	.db -39, -48, -56, -62, -66, -69, -70, -71, -70, -70, -70, -71, -72, -75, -77, -79
	.db -80, -78, -75, -69, -61, -50, -38, -26, -13, -2, 7, 15, 21, 25, 28, 30
	.db 33, 36, 40, 44, 49, 55, 59, 61, 62, 60, 56, 49, 42, 34, 27, 22
	.db 20, 21, 25, 33, 42, 52, 63, 72, 80, 86, 89, 90, 89, 87, 84, 81
	.db 79, 77, 75, 73, 72, 69, 66, 61, 55, 48, 39, 31, 22, 14, 6, 0
	.db -5, -10, -14, -17, -20, -22, -24, -26, -28, -29, -30, -31, -32, -33, -34, -35
	.db -36, -37, -39, -41, -44, -48, -52, -57, -63, -70, -77, -85, -94, -102, -110, -116
	.db -121, -124, -124, -121, -116, -109, -99, -89, -78, -68, -60, -53, -48, -45, -43, -43
	.db -43, -42, -41, -40, -37, -33, -29, -25, -22, -19, -17, -16, -15, -14, -12, -9
	.db -4, 2, 11, 22, 34, 46, 57, 67, 75, 81, 84, 86, 87, 86, 86, 85
	.db 85, 85, 86, 86, 87, 86, 84, 82, 78, 73, 69, 64, 60, 58, 56, 57
	.db 59, 62, 66, 70, 75, 79, 83, 86, 89, 91, 92, 92, 92, 90, 86, 81
	.db 73, 63, 52, 38, 24, 9, -5, -19, -31, -42, -50, -57, -61, -64, -66, -67
	.db -68, -68, -68, -67, -66, -64, -61, -56, -52, -46, -41, -36, -32, -29, -28, -28
	.db -30, -34, -39, -44, -50, -56, -62, -68, -74, -79, -84, -90, -95, -100, -104, -107
	.db -108, -108, -107, -103, -98, -92, -84, -76, -67, -58, -49, -40, -32, -24, -16, -8
	.db -1, 5, 11, 16, 20, 22, 22, 21, 18, 14, 10, 5, 1, -1, -3, -3
	.db -2, 0, 1, 5, 9, 12, 16, 20, 24, 28, 34, 40, 47, 55, 64, 72
	.db 81, 89, 96, 101, 105, 108, 108, 108, 107, 105, 103, 101, 100, 98, 96, 93
	.db 90, 87, 83, 79, 75, 70, 66, 62, 59, 56, 55, 53, 52, 51, 50, 48
	.db 46, 44, 42, 39, 37, 34, 31, 28, 24, 20, 15, 10, 3, -3, -12, -21
	.db -30, -39, -48, -57, -65, -72, -79, -85, -90, -95, -99, -103, -106, -109, -110, -112
	.db -112, -112, -111, -110, -109, -107, -106, -105, -104, -104, -103, -102, -101, -99, -97, -94
	.db -90, -86, -82, -78, -75, -72, -69, -67, -65, -63, -61, -58, -54, -50, -44, -38
	.db -31, -23, -16, -10, -4, 1, 6, 10, 14, 18, 23, 27, 32, 38, 43, 49
	.db 54, 59, 63, 67, 70, 73, 75, 77, 79, 81, 83, 85, 88, 91, 93, 96
	.db 99, 101, 103, 105, 107, 109, 110, 111, 112, 112, 112, 111, 109, 107, 104, 100
	.db 96, 92, 88, 83, 79, 75, 70, 65, 60, 55, 49, 43, 36, 30, 24, 18
	.db 12, 7, 3, 0, -3, -6, -8, -11, -14, -17, -20, -24, -28, -32, -37, -42
	.db -46, -51, -55, -59, -63, -67, -71, -75, -79, -82, -85, -88, -91, -93, -95, -97
	.db -98, -100, -101, -102, -103, -104, -104, -104, -103, -101, -99, -96, -93, -89, -86, -83
	.db -80, -78, -76, -75, -74, -73, -72, -71, -70, -68, -65, -63, -59, -56, -52, -48
	.db -44, -40, -35, -30, -24, -18, -11, -4, 3, 11, 19, 27, 34, 41, 48, 53
	.db 58, 63, 67, 71, 75, 79, 82, 86, 89, 92, 94, 95, 96, 97, 98, 98
	.db 99, 100, 101, 103, 105, 108, 110, 112, 114, 115, 115, 114, 113, 110, 107, 103
	.db 99, 95, 91, 87, 83, 79, 74, 70, 65, 59, 53, 47, 41, 35, 29, 24
	.db 19, 14, 11, 7, 4, 1, -1, -5, -9, -14, -19, -25, -31, -37, -43, -50
	.db -56, -62, -68, -73, -78, -83, -88, -92, -97, -101, -105, -109, -112, -115, -117, -119
	.db -120, -121, -120, -119, -118, -116, -114, -112, -109, -106, -104, -101, -99, -96, -94, -91
	.db -88, -85, -81, -78, -74, -70, -66, -62, -57, -53, -49, -45, -41, -37, -32, -27
	.db -22, -16, -10, -4, 1, 8, 14, 20, 25, 31, 36, 41, 46, 50, 55, 60
	.db 64, 69, 73, 77, 81, 85, 88, 90, 93, 95, 97, 99, 101, 103, 106, 109
	.db 112, 115, 118, 121, 123, 125, 126, 126, 125, 125, 123, 121, 119, 117, 115, 112
	.db 110, 107, 103, 99, 94, 89, 83, 76, 69, 62, 54, 47, 39, 31, 24, 16
	.db 9, 2, -4, -11, -18, -25, -31, -38, -44, -51, -57, -62, -68, -73, -79, -84
	.db -88, -93, -97, -102, -106, -109, -113, -116, -119, -121, -123, -124, -125, -126, -127, -127
	.db -127, -127, -126, -125, -125, -123, -122, -120, -118, -116, -114, -111, -107, -104, -100, -95
	.db -91, -86, -80, -75, -69, -63, -58, -52, -46, -40, -34, -28, -22, -16, -10, -4

wt_loop: ; Sustain area
	.db 0, 5, 11, 17, 23, 28, 34, 39, 45, 50, 55, 60, 65, 69, 74, 78
	.db 82, 85, 89, 92, 95, 98, 100, 102, 104, 106, 107, 109, 109, 110, 110, 111
	.db 110, 110, 109, 108, 107, 106, 104, 102, 100, 98, 95, 93, 90, 87, 83, 80
	.db 76, 72, 68, 64, 59, 55, 50, 46, 41, 36, 31, 26, 21, 15, 10, 5
	.db 0, -5, -10, -15, -21, -26, -31, -36, -41, -46, -50, -55, -59, -64, -68, -72
	.db -76, -80, -83, -87, -90, -93, -95, -98, -100, -102, -104, -106, -107, -108, -109, -110
	.db -110, -111, -110, -110, -109, -109, -107, -106, -104, -102, -100, -98, -95, -92, -89, -85
	.db -82, -78, -74, -69, -65, -60, -55, -50, -45, -39, -34, -28, -23, -17, -11, -5
wt_end:

*/
